/*===- TableGen'erated file -------------------------------------*- C++ -*-===*\
|*                                                                            *|
|* Op Declarations                                                            *|
|*                                                                            *|
|* Automatically generated file, do not edit!                                 *|
|* From: DLTITransformOps.td                                                  *|
|*                                                                            *|
\*===----------------------------------------------------------------------===*/

namespace mlir {
namespace transform {
/// Return attribute (as param) associated to key via DTLI
/// This op queries data layout and target information associated to payload
///     IR by way of the DLTI dialect.
/// 
///     A lookup is performed for the given `keys` at `target` op - or its closest
///     interface-implementing ancestor - by way of the `DLTIQueryInterface`, which
///     returns an attribute for a key. Each key should be either a (quoted) string
///     or a type. If more than one key is provided, the lookup continues
///     recursively, now on the returned attributes, with the condition that these
///     implement the above interface. For example if the payload IR is
/// 
///     ```
///     module attributes {#dlti.map = #dlti.map<#dlti.dl_entry<"A",
///                                      #dlti.map<#dlti.dl_entry<"B", 42: int>>>} {
///       func.func private @f()
///     }
///     ```
///     and we have that `%func` is a Tranform handle to op `@f`, then
///     `transform.dlti.query ["A", "B"] at %func` returns 42 as a param and
///     `transform.dlti.query ["A"] at %func` returns the `#dlti.map` attribute
///     containing just the key "B" and its value. Using `["B"]` or `["A","C"]` as
///     `keys` will yield an error.
/// 
///     #### Return modes
/// 
///     When successful, the result, `associated_attr`, associates one attribute as
///     a param for each op in `target`'s payload.
/// 
///     If the lookup fails - as no DLTI attributes/interfaces are found or entries
///     with the right names are missing - a silenceable failure is returned.
class QueryOp;
} // namespace transform
} // namespace mlir
#ifdef GET_OP_CLASSES
#undef GET_OP_CLASSES

namespace mlir {
namespace transform {

//===----------------------------------------------------------------------===//
// ::mlir::transform::QueryOp declarations
//===----------------------------------------------------------------------===//

namespace detail {
class QueryOpGenericAdaptorBase {
public:
  struct Properties {
    using keysTy = ::mlir::ArrayAttr;
    keysTy keys;

    auto getKeys() const {
      auto &propStorage = this->keys;
      return ::llvm::cast<::mlir::ArrayAttr>(propStorage);
    }
    void setKeys(const ::mlir::ArrayAttr &propValue) {
      this->keys = propValue;
    }
    bool operator==(const Properties &rhs) const {
      return 
        rhs.keys == this->keys &&
        true;
    }
    bool operator!=(const Properties &rhs) const {
      return !(*this == rhs);
    }
  };
protected:
  ::mlir::DictionaryAttr odsAttrs;
  ::std::optional<::mlir::OperationName> odsOpName;
  Properties properties;
  ::mlir::RegionRange odsRegions;
public:
  QueryOpGenericAdaptorBase(::mlir::DictionaryAttr attrs, const Properties &properties, ::mlir::RegionRange regions = {}) : odsAttrs(attrs), properties(properties), odsRegions(regions) {  if (odsAttrs)
      odsOpName.emplace("transform.dlti.query", odsAttrs.getContext());
  }

  QueryOpGenericAdaptorBase(QueryOp op);

  std::pair<unsigned, unsigned> getODSOperandIndexAndLength(unsigned index, unsigned odsOperandsSize) {
    return {index, 1};
  }

  const Properties &getProperties() {
    return properties;
  }

  ::mlir::DictionaryAttr getAttributes() {
    return odsAttrs;
  }

  ::mlir::ArrayAttr getKeysAttr() {
    auto attr = ::llvm::cast<::mlir::ArrayAttr>(getProperties().keys);
    return attr;
  }

  ::mlir::ArrayAttr getKeys();
};
} // namespace detail
template <typename RangeT>
class QueryOpGenericAdaptor : public detail::QueryOpGenericAdaptorBase {
  using ValueT = ::llvm::detail::ValueOfRange<RangeT>;
  using Base = detail::QueryOpGenericAdaptorBase;
public:
  QueryOpGenericAdaptor(RangeT values, ::mlir::DictionaryAttr attrs, const Properties &properties, ::mlir::RegionRange regions = {}) : Base(attrs, properties, regions), odsOperands(values) {}

  QueryOpGenericAdaptor(RangeT values, ::mlir::DictionaryAttr attrs, ::mlir::OpaqueProperties properties, ::mlir::RegionRange regions = {}) : QueryOpGenericAdaptor(values, attrs, (properties ? *properties.as<Properties *>() : Properties{}), regions) {}

  QueryOpGenericAdaptor(RangeT values, ::mlir::DictionaryAttr attrs = nullptr) : QueryOpGenericAdaptor(values, attrs, Properties{}, {}) {}

  QueryOpGenericAdaptor(RangeT values, const QueryOpGenericAdaptorBase &base) : Base(base), odsOperands(values) {}

  template <typename LateInst = QueryOp, typename = std::enable_if_t<std::is_same_v<LateInst, QueryOp>>>
  QueryOpGenericAdaptor(RangeT values, LateInst op) : Base(op), odsOperands(values) {}

  std::pair<unsigned, unsigned> getODSOperandIndexAndLength(unsigned index) {
    return Base::getODSOperandIndexAndLength(index, odsOperands.size());
  }

  RangeT getODSOperands(unsigned index) {
    auto valueRange = getODSOperandIndexAndLength(index);
    return {std::next(odsOperands.begin(), valueRange.first),
             std::next(odsOperands.begin(), valueRange.first + valueRange.second)};
  }

  ValueT getTarget() {
    return (*getODSOperands(0).begin());
  }

  RangeT getOperands() {
    return odsOperands;
  }

private:
  RangeT odsOperands;
};
class QueryOpAdaptor : public QueryOpGenericAdaptor<::mlir::ValueRange> {
public:
  using QueryOpGenericAdaptor::QueryOpGenericAdaptor;
  QueryOpAdaptor(QueryOp op);

  ::llvm::LogicalResult verify(::mlir::Location loc);
};
class QueryOp : public ::mlir::Op<QueryOp, ::mlir::OpTrait::ZeroRegions, ::mlir::OpTrait::OneResult, ::mlir::OpTrait::OneTypedResult<::mlir::transform::TransformParamTypeInterface>::Impl, ::mlir::OpTrait::ZeroSuccessors, ::mlir::OpTrait::OneOperand, ::mlir::OpTrait::OpInvariants, ::mlir::BytecodeOpInterface::Trait, ::mlir::transform::TransformOpInterface::Trait, ::mlir::transform::TransformEachOpTrait, ::mlir::MemoryEffectOpInterface::Trait> {
public:
  using Op::Op;
  using Op::print;
  using Adaptor = QueryOpAdaptor;
  template <typename RangeT>
  using GenericAdaptor = QueryOpGenericAdaptor<RangeT>;
  using FoldAdaptor = GenericAdaptor<::llvm::ArrayRef<::mlir::Attribute>>;
  using Properties = FoldAdaptor::Properties;
  static ::llvm::ArrayRef<::llvm::StringRef> getAttributeNames() {
    static ::llvm::StringRef attrNames[] = {::llvm::StringRef("keys")};
    return ::llvm::ArrayRef(attrNames);
  }

  ::mlir::StringAttr getKeysAttrName() {
    return getAttributeNameForIndex(0);
  }

  static ::mlir::StringAttr getKeysAttrName(::mlir::OperationName name) {
    return getAttributeNameForIndex(name, 0);
  }

  static constexpr ::llvm::StringLiteral getOperationName() {
    return ::llvm::StringLiteral("transform.dlti.query");
  }

  std::pair<unsigned, unsigned> getODSOperandIndexAndLength(unsigned index) {
    return {index, 1};
  }

  ::mlir::Operation::operand_range getODSOperands(unsigned index) {
    auto valueRange = getODSOperandIndexAndLength(index);
    return {std::next(getOperation()->operand_begin(), valueRange.first),
             std::next(getOperation()->operand_begin(), valueRange.first + valueRange.second)};
  }

  ::mlir::TypedValue<::mlir::transform::TransformHandleTypeInterface> getTarget() {
    return ::llvm::cast<::mlir::TypedValue<::mlir::transform::TransformHandleTypeInterface>>(*getODSOperands(0).begin());
  }

  ::mlir::OpOperand &getTargetMutable() {
    auto range = getODSOperandIndexAndLength(0);
    return getOperation()->getOpOperand(range.first);
  }

  std::pair<unsigned, unsigned> getODSResultIndexAndLength(unsigned index) {
    return {index, 1};
  }

  ::mlir::Operation::result_range getODSResults(unsigned index) {
    auto valueRange = getODSResultIndexAndLength(index);
    return {std::next(getOperation()->result_begin(), valueRange.first),
             std::next(getOperation()->result_begin(), valueRange.first + valueRange.second)};
  }

  ::mlir::TypedValue<::mlir::transform::TransformParamTypeInterface> getAssociatedAttr() {
    return ::llvm::cast<::mlir::TypedValue<::mlir::transform::TransformParamTypeInterface>>(*getODSResults(0).begin());
  }

  static ::llvm::LogicalResult setPropertiesFromAttr(Properties &prop, ::mlir::Attribute attr, ::llvm::function_ref<::mlir::InFlightDiagnostic()> emitError);
  static ::mlir::Attribute getPropertiesAsAttr(::mlir::MLIRContext *ctx, const Properties &prop);
  static llvm::hash_code computePropertiesHash(const Properties &prop);
  static std::optional<mlir::Attribute> getInherentAttr(::mlir::MLIRContext *ctx, const Properties &prop, llvm::StringRef name);
  static void setInherentAttr(Properties &prop, llvm::StringRef name, mlir::Attribute value);
  static void populateInherentAttrs(::mlir::MLIRContext *ctx, const Properties &prop, ::mlir::NamedAttrList &attrs);
  static ::llvm::LogicalResult verifyInherentAttrs(::mlir::OperationName opName, ::mlir::NamedAttrList &attrs, llvm::function_ref<::mlir::InFlightDiagnostic()> emitError);
  static ::llvm::LogicalResult readProperties(::mlir::DialectBytecodeReader &reader, ::mlir::OperationState &state);
  void writeProperties(::mlir::DialectBytecodeWriter &writer);
  ::mlir::ArrayAttr getKeysAttr() {
    return ::llvm::cast<::mlir::ArrayAttr>(getProperties().keys);
  }

  ::mlir::ArrayAttr getKeys();
  void setKeysAttr(::mlir::ArrayAttr attr) {
    getProperties().keys = attr;
  }

  static void build(::mlir::OpBuilder &odsBuilder, ::mlir::OperationState &odsState, ::mlir::Type associated_attr, ::mlir::Value target, ::mlir::ArrayAttr keys);
  static void build(::mlir::OpBuilder &odsBuilder, ::mlir::OperationState &odsState, ::mlir::TypeRange resultTypes, ::mlir::Value target, ::mlir::ArrayAttr keys);
  static void build(::mlir::OpBuilder &, ::mlir::OperationState &odsState, ::mlir::TypeRange resultTypes, ::mlir::ValueRange operands, ::llvm::ArrayRef<::mlir::NamedAttribute> attributes = {});
  static void build(::mlir::OpBuilder &, ::mlir::OperationState &odsState, ::mlir::TypeRange resultTypes, ::mlir::ValueRange operands, const Properties &properties, ::llvm::ArrayRef<::mlir::NamedAttribute> discardableAttributes = {});
  ::llvm::LogicalResult verifyInvariantsImpl();
  ::llvm::LogicalResult verifyInvariants();
  void getEffects(::llvm::SmallVectorImpl<::mlir::SideEffects::EffectInstance<::mlir::MemoryEffects::Effect>> &effects);
  static ::mlir::ParseResult parse(::mlir::OpAsmParser &parser, ::mlir::OperationState &result);
  void print(::mlir::OpAsmPrinter &_odsPrinter);
private:
  ::mlir::StringAttr getAttributeNameForIndex(unsigned index) {
    return getAttributeNameForIndex((*this)->getName(), index);
  }

  static ::mlir::StringAttr getAttributeNameForIndex(::mlir::OperationName name, unsigned index) {
    assert(index < 1 && "invalid attribute index");
    assert(name.getStringRef() == getOperationName() && "invalid operation name");
    assert(name.isRegistered() && "Operation isn't registered, missing a "
          "dependent dialect loading?");
    return name.getAttributeNames()[index];
  }

public:
  ::mlir::DiagnosedSilenceableFailure applyToOne(
      ::mlir::transform::TransformRewriter &rewriter,
      ::mlir::Operation *target,
      ::mlir::transform::ApplyToEachResultList &results,
      TransformState &state);
};
} // namespace transform
} // namespace mlir
MLIR_DECLARE_EXPLICIT_TYPE_ID(::mlir::transform::QueryOp)


#endif  // GET_OP_CLASSES

